import java.util.LinkedList;

/**
 * Denne klassen representerer en node som skal visualiseres i 3D
 * Den inneholder posisjonen til noden som skal tegnes opp,
 * en tabell over strenger, og en liste over koblinger til andre noder 							
 *
 * @author Ole Kristian Hvaale, 1. Februar 2006 
 * -----------------------------------------------------
 * Sist oppdatert: 2006/5/12
 * av: Ole Kristian Hvaale
 * Endringer: ryddet i kode og oppdatert Javadoc
 * Lagt til indeks til DocumentItems for  kunne lettere oppdatere
 * data for frekvensbilde.
 */

public class DocumentItem
{
	private float posx;
	private float posy;
	private float posz;
	private String[] documentnames;
	private LinkedList connections= new LinkedList();
	private int indeks;
	/***
	 *  
	 * Oppretter et DocumentItem og genererer tilfeldige verdier for posisjonene x y og z
	 * 
	 */
	public DocumentItem(int test)
	{
		indeks =test;
		int number;
		posx = (float)Math.random()*6;//random float fra 0f til 6f
		number = (int) (Math.random()*2);// set til at tallene blir random -6f til 6f
		if(number == 0)
		{
			posx = posx*-1;
		}
					
		posy = (float)Math.random()*6;
		number = (int) (Math.random()*2);
		if(number==0)
		{
			posy =posy*-1;
		}
			
		posz = (float)Math.random()*6;
		number = (int) (Math.random()*2);
		if(number==0)
		{
			posz =posz*-1;
		}		
		
	}
	/**
	 * Metoden srger for at man kan legge til tabell over URLer til noden
	 * 
	 * @param documentarray String tabell som inneholder URLer
	 */
	public void setDocumentlist(String[] documentarray)
	{
		documentnames = documentarray;
	}
	/**
	 * Henter ut tabell over URLer
	 * 
	 * @return Returnerer tabellen med URL`ne som tilhrer noden
	 */
	public String[] getDocumentStrings()
	{
		return documentnames;
	}
	/**
	 * Henter ut x koordinaten til noden
	 * 
	 * @return Returnerer x koordinaten til noden
	 */
	public float getPosx()
	{
		return posx;
	}
	/**
	 * Henter ut y koordinaten til noden
	 * 
	 * @return Returnerer y koordinaten til noden
	 */
	public float getPosy()
	{
		return posy;	
	}
	/**
	 * Henter ut z koordinaten til noden
	 * 
	 * @return Returnerer z koordinaten til noden
	 */	
	public float getPosz()
	{
		return posz;
	}
	/**
	 * 	Setter x koordinaten til noden
	 * 
	 */
	public void setPosx(float pos_x)
	{
		posx=pos_x;
	}
	/**
	 * 	Setter y koordinaten til noden
	 * 
	 */
	public void setPosy(float pos_y)
	{
		posy=pos_y;
	}
	/**
	 * 	Setter z koordinaten til noden
	 * 
	 */
	public void setPosz(float pos_z)
	{
		posz=pos_z;
	}
	/**
	 * Oppretter en kobling til node som det skal tegnes kant mellom
	 * 
	 * @param node Oppretter en kobling mellom nodene
	 */	
	public void addConnection(DocumentItem node)
	{
		connections.add(node);
	}
	/**
	 * Returnerer node fra nodelisten basert p spesifisert indeks
	 * 
	 * @param elementnr Spesifiserer hvilken indeks man vil hente ut node
	 * @return DocumentItem Objektet som er lagret p spesifisert indeks
	 */
	public DocumentItem getConnectionNode(int elementnr)
	{
		if(elementnr > getSize())
		{
			return null;
		}
		else
		{
			DocumentItem denne;
			denne = (DocumentItem)connections.get(elementnr);
			return denne;
		}
	}
	/**
	 * Henter ut antall koblinger knyttet til noden
	 * 
	 * @return Returnerer antall koblinger nvrende Node har til andre noder
	 */
	public int getSize()
	{
		int size;
		size = connections.size();
		return size;
	}
	/**
	 * Henter ut indeksen til noden. indeksen er basert p rekkeflgen
	 * nodene er laget.
	 * 
	 * @return indeks til nverende node.
	 */
	public int getindeks()
	{
		return indeks;
	}
	
}