import java.util.LinkedList;
import java.util.StringTokenizer;

import java.io.*;

/**
 * Klassen har som oppgave  lese inn data som skal visualiseres
 * dataene leses inn i en liste og lagres i en instans av klassen
 * EqualityMatrix
 * 
 * @author Ole Kristian Hvaale, 1. Februar 2006 
 * -----------------------------------------------------
 * Sist oppdatert: 2006/7/12
 * av: Ole Kristian Hvaale
 * Lagt til metodene saveBrowseHistory(), openBrowsehistory() oppdatert Javadoc
 *
 */
public class Filereader
{
	private LinkedList documentitemlist;//liste over de nodene som skal visualiseres
	private EqualityMatrix equalitymatrix;
	private int[] frekarray;
	private int numberoftokensinline;
	/**
	 * Leser inn likhetsmatrisen oppretter noder og lagrer likhetsmatrisen
	 * 
	 * @param inputfile Filen som inneholder likhetsmatrisen 
	 */
	public void readURLFile(String inputfile)
	{
		documentitemlist = new LinkedList();
		StringTokenizer tokenizer;
		String line;
		String file = inputfile;
		int count=0;
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);
			
			line = inFile.readLine();
			tokenizer = new StringTokenizer(line);
			
			numberoftokensinline =tokenizer.countTokens();

			//oppretter datanoder
			for(int i =0; i<numberoftokensinline; i++)
			{
				
				try
				{
					String docname =tokenizer.nextToken();
					DocumentItem docitem = new DocumentItem(count);

					String[] tmparray = new String[1];
					tmparray[0]= docname;
					docitem.setDocumentlist(tmparray);
					documentitemlist.add(docitem);
					count++;		
				}
				catch(NumberFormatException exception)
				{
					System.out.println("Error in input. Line ignored");
					System.out.println(line);
				}
				
			}

		} 
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");			
		}
		catch(IOException exception)
		{
			System.out.println(exception);
		}		
		
	}
	/**
	 * Lagrer plasseringen spring embedderen har gitt nodene.
	 * Gjres for  f et konstant bilde, brukeren vil f samme himmel opp hver gang.
	 * 
	 * @param list liste over noder der plasseringene til nodene kan leses fra
	 */
	public void writeGraphData(LinkedList list, String map)
	{
	   File graphdatafile;
	   FileWriter     writer = null;
	   BufferedWriter buffer = null;
	   try
	   {        
	    	graphdatafile = new File(map+"/"+"graphdata.dat");
	    	writer = new FileWriter(graphdatafile);
	    	buffer = new BufferedWriter(writer);
	    	for(int i =0; i<list.size(); i++)
	    	{
	    		DocumentItem test;
	    		test=(DocumentItem)list.get(i);
				buffer.write(test.getPosx()+" "+test.getPosy()+" "+test.getPosz());

				buffer.newLine();
	    	}
	    	
	    	buffer.close();
		
		}
		catch( IOException e )
		{
			System.out.println("Feil under Skriving til fil!:"+e.getMessage());
		}
	}
	/**
	 * Leser inn data for nodenes plassering og lagrer dataene i de nodene de tilhrer
	 * 
	 * @param list liste over nodene som skal oppdateres med plasserings koordinater
	 * 
	 * @return true hvis filen eksisterer, false hvis den ikke eksisterer.
	 */
	public boolean readGraphData(LinkedList list, String map)
	{
		StringTokenizer tokenizer;
		String line;
		String file = map+"/"+"graphdata.dat";
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);		
			line = inFile.readLine();
			tokenizer = new StringTokenizer(line);
			for(int i=0; i<list.size(); i++)
			{
				DocumentItem tempitem =(DocumentItem)list.get(i);
				tempitem.setPosx(Float.parseFloat(tokenizer.nextToken()));
				tempitem.setPosy(Float.parseFloat(tokenizer.nextToken()));
				tempitem.setPosz(Float.parseFloat(tokenizer.nextToken()));
				
				if(i==list.size()-1)
				{
				}
				else
				{
					line = inFile.readLine();
					tokenizer = new StringTokenizer(line);
				}
			}
			inFile.close();
			
			return true;
		} 
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");
			return false;
		}
		catch(IOException exception)
		{
			System.out.println(exception);
			return false;
		}			
			
	}
	/**
	 * Leser inn dataene fra likhetsmatrise filen og lagrer de i klassen EqualityMatrix();
	 * 
	 * 
	 * @param inputfile
	 */
	public void readEqualityFile(String inputfile)
	{
		equalitymatrix= new EqualityMatrix();
		StringTokenizer tokenizer;
		String line;
		String file = inputfile;	
			
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);		
			line = inFile.readLine();
			int count=0;//hjelpe teller for indeksering
						
			while(numberoftokensinline != count)
			{
				count++;
				tokenizer = new StringTokenizer(line);
						
				try
				{
					float[] equalityarrayfornode = new float[numberoftokensinline];
			
					for(int i=0; i<count; i++)
					{
						float tmpfloat;
						tmpfloat=Float.parseFloat(tokenizer.nextToken());
						equalityarrayfornode[i] = tmpfloat;
						
					}
					
					equalitymatrix.addArray(equalityarrayfornode);
					tokenizer = new StringTokenizer(line);
				
				}
				catch(NumberFormatException exception)
				{
					System.out.println("Error in input. Line ignored");
					System.out.println(line);
					
				}
				line = inFile.readLine();
							
			}
			
			inFile.close();
		
		} 
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");			
		}
		catch(IOException exception)
		{
			System.out.println(exception);
		}		
			
	}	
	/***
	 *  Leser inn navn p "hovedklynge" og URL`ene den er knyttet til,
	 * 	lagrer URL`ene hos noden som eier dataene
	 * 
	 * @param inputfile
	 */
	public void readDocumentsForNodeFile(String inputfile)
	{
		
		StringTokenizer tokenizer;
		String line;
		String file = inputfile;
		
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);
			
			line = inFile.readLine();
			
			int counter=0;
			while(line != null)
			{
				tokenizer = new StringTokenizer(line);
				int numberoftokensinline = tokenizer.countTokens();
				
				String[] tmparray = new String[numberoftokensinline];
				for(int i=0; i<numberoftokensinline; i++)
				{
					tmparray[i]=tokenizer.nextToken();
								
				}
				DocumentItem tmpdocitem =(DocumentItem)documentitemlist.get(counter);
				tmpdocitem.setDocumentlist(tmparray);
				line = inFile.readLine();
				counter++;
				
			}
						
		}
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");			
		}
		catch(IOException exception)
		{
			System.out.println(exception);
		}		
			
	}
	/**
	 * Leser inn data for frekvensfil. oppdaterer en intern tabell i klassen Filereader som kan
	 * hentes ut av andre klasser.
	 * tabell inneholder antall besk brukeren har beskt hver node.
	 * 
	 * @return true hvis filen eksisterer, false hvis filen ikke ekisterer
	 */
	public boolean readFrekvensFile(String nameoftestmap)
	{
		StringTokenizer tokenizer;
		String line;
		String file =nameoftestmap+"/"+"frek.dat";
		
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);		
			line = inFile.readLine();
			tokenizer = new StringTokenizer(line);
			int numberoftokensinline = tokenizer.countTokens();
			frekarray = new int[numberoftokensinline];
			
			for(int i=0; i<numberoftokensinline; i++)
			{
				frekarray[i]= Integer.parseInt(tokenizer.nextToken());
						
			}
			inFile.close();
			
			
			return true;
		} 
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");
			return false;
		}
		catch(IOException exception)
		{
			System.out.println(exception);
			return false;
		}			
			
	}
	/**
	 * Henter ut listen over noder
	 * 
	 * @return Returnerer listen som inneholder objektene DocumentItem
	 */
	public LinkedList getDocumentItemList()
	{
		return documentitemlist;
	}
	/**
	 * Henter ut listen EqualityMatrix 
	 * 
	 * @return Returnerer listen matrix som inneholder koblingen mellom nodene
	 */
	public EqualityMatrix getEqualityMatrix()
	{
		return equalitymatrix;
	}
	/**
	 * Henter ut tabellen over frekvensdata
	 * 
	 * @return tabell over frekvens man har beskt nodene.
	 */
	public int[] getFrekArray()
	{
		return frekarray;
	}
	/**
	 * Skriver Frekvensdata til fil.
	 * 
	 * @param frek tabellen over antall ganger man har beskt hver node
	 */
	public void writeFrekdata(int[] frek, String nameoftestmap)
	{
	   File graphdatafile;
	   FileWriter     writer = null;
	   BufferedWriter buffer = null;
	   try
	   {        
	    	graphdatafile    = new File(nameoftestmap+"/"+"frek.dat");
	    	writer = new FileWriter(graphdatafile);
	    	buffer = new BufferedWriter(writer);
	    	for(int i =0; i<frek.length; i++)
	    	{
				buffer.write(frek[i]+" ");
		
			}
	    	
	    	buffer.close();
		
	   }
	   catch( IOException e )
	   {
		   System.out.println("Feil under Skriving til fil!:"+e.getMessage());
	   }
	}
	/**
	 * pner browsehistorie som er lagret p fil leser inn  oppdaterer sesjons historien
	 * 
	 * 
	 * @param filn filen som inneholder frekvensdata
	 * @return resurnere en liste over noder som beskriver hvilke noder som er i browse historien
	 */
	public LinkedList openBrowseHistory(String filname)
	{
		LinkedList history= new LinkedList();
		StringTokenizer tokenizer;
		String line;
		String file = filname;
		
		try
		{
			FileReader filereader = new FileReader(file);
			BufferedReader inFile = new BufferedReader(filereader);		
			line = inFile.readLine();
			tokenizer = new StringTokenizer(line);
			int numberoftokensinline = tokenizer.countTokens();
						
			for(int i=0; i<numberoftokensinline; i++)
			{
				StackNode stacknode = new StackNode();
				int nr =Integer.parseInt(tokenizer.nextToken());
				stacknode.setNodeNr(nr);
				history.add(stacknode);		
			}
			inFile.close();
			
		} 
		catch(FileNotFoundException exception)
		{
			System.out.println("The file " + file+ " was not found.");
			
		}
		catch(IOException exception)
		{
			System.out.println(exception);
			
		}			
	
		return history;
		
	}
	/**
	 * Lagrer browsehistorien til fil.
	 * 
	 * 
	 * @param filn navnet p filen som det skal lageres frekvensdata til 
	 * @param ll listen over sesjons histoien som skal lagres
	 */
	public void saveBrowseHistory(String filn, LinkedList ll)
	{
		String filename = filn+".path";
		File browsefile;
		FileWriter     writer = null;
		BufferedWriter buffer = null;
		try
		{        
			browsefile    = new File(filename);
	    	writer = new FileWriter(browsefile);
	    	buffer = new BufferedWriter(writer);
	    	
	    	for(int i =0; i<ll.size(); i++)
	    	{
	    		StackNode tmp =(StackNode)ll.get(i);
				buffer.write(tmp.getNodeNr()+" ");
			}
	    	
	    	buffer.close();
		
		}
		catch( IOException e )
		{
		System.out.println("Feil under Skriving til fil!:"+e.getMessage());
		}
		}	
			
	}

