import java.util.LinkedList;
/**
 * Beregner nye x, y z verdier for nodene som skal tegnes opp i Java3d
 * Klassen er laget med modifikasjoner av bl.a. Fruchterman lagt til av Tore Vestues
 * videre modifikasjoner av David Kaasen 2004-05-21
 * 
 * @author Ole Kristian Hvaale, 1. Februar 2006 
 * -----------------------------------------------------
 * Sist oppdatert: 2006/5/11
 * av: Ole Kristian Hvaale
 * Endringer: forandret antall iterasjoner som slyfen gjennomfrer
 * for  forbedre plasseringen av nodene.
 * 
 */
public class SpringEmbedder
{
	private EqualityMatrix equalitymatrix;
	private LinkedList listofnodes;
	private final int ITERATIONS = 250;
	private int numberofnodes;
	/**
	 * @param equmatrix Likehetsmatrisen over nodene
	 * @param list	Liste over nodene som skal inng i spring embedderen
	 * 
	 */
	public SpringEmbedder(EqualityMatrix equmatrix, LinkedList list)
	{
		equalitymatrix = equmatrix;
		listofnodes = list;
		
	}
	/**
	 * Utfrer spring embedder algoritmen
	 */	
	public void preformSpringEmbedder()
	{
		boolean finished=false;
		numberofnodes = listofnodes.size();

		if(numberofnodes<=1)
		{
			//ingen noder eller bare 1 trenger ikke utfre noe spring embedder
		}
		else
		{
			
			//beregner energi
		     float energySum = 0.0f;
		     float prevEnergySum = 0.0f;
		     float tmpfloat;
		     // beregner total energi for alle nodene som skal visualiseres
		     for (int i = 0; i < numberofnodes; i++)
		     {
		         energySum += getEnergy(i);
 
		     }
//1. utskrift brukt for  teste at energien reduseres.		     
//		     System.out.println("Start energi " + energySum); 
		     // gjr energien mindre
		      float[] bestDir = new float[3];
		      float currentEnergy = 0.0f;		     
		      DocumentItem gldNode = null;

		      for (int step = 1; step <= ITERATIONS; step++)
		      {
			      if(!finished)
			      {			    	  
		    	  	//	for all non-fixed nodes: minimize energy
			          prevEnergySum = energySum;
			          energySum = 0.0f;
			          for (int i = 0; i < numberofnodes; i++)
			          {
	
			        	  gldNode = (DocumentItem) listofnodes.get(i);
			              float oldEnergy = getEnergyDir(i, bestDir);
	
			              gldNode.setPosx(gldNode.getPosx() + bestDir[0]);
			              gldNode.setPosy(gldNode.getPosy() + bestDir[1]);
			              gldNode.setPosz(gldNode.getPosz() + bestDir[2]);		        	  
			              float curEnergy = getEnergy(i);
			              
			               if (curEnergy >= oldEnergy)
			               {
//			                  System.out.println("Setter posisjonen tilbake...");
			                  gldNode.setPosx(gldNode.getPosx() - bestDir[0]);
			                  gldNode.setPosy(gldNode.getPosy() - bestDir[1]);
			                  gldNode.setPosz(gldNode.getPosz() - bestDir[2]);
			                  curEnergy = oldEnergy;
			               }
	
			               energySum += curEnergy;
			        	  
			          }
			          tmpfloat = prevEnergySum/energySum;
			          
//1. utskrift brukt for  teste at energien reduseres.			          
//			          System.out.println("Energi: " + energySum + ",\tf: " + tmpfloat);
			          if (step > 7 && tmpfloat > 0.9996f)
			          {
	//		             System.out.println(""+step+" For liten forskjell, stanser her.");
			             finished=true;
	  			      
			          }
		  
			      }
			      currentEnergy = energySum;
			      
		      }
	
		}

	}
	/**
	 * Beregner energien til en Node
	 * @param index noden man skal beregne energi til
	 * @return	Energien til noden
	 * 
	 */
	public float getEnergy(int index)
	{
	      float energy = 0.0f;
	      float dist;          // Euklidean distanse til annen node
	      float tmp;
	      boolean finished = false;
	      
	      for (int i = 0; i < numberofnodes; i++)
	      {
	    	 if(!finished)
	    	 {	    	  
		         if (i != index)
		         {
		            dist = getDist(i, index);//kalulerer distanse mellom to noder
		            if (dist > 0.00001f)
		            {
		               tmp = dist*equalitymatrix.getEquality(index, i);
		               energy += tmp*tmp*tmp / 3 - (float)Math.log(dist);
		            }
		            else
		            {
		               energy = Float.MAX_VALUE;
		               finished = true;
		            }
		            
		          }
		         
	    	 }
	  	         
	      }
	      return energy;		
		
	}
	/**
	 * Beregner Distansen mellom de spesifiserte nodene
	 * @param i1 Node1
	 * @param i2 Node2
	 * @return Distansen mellom nodene
	 */
	protected float getDist(int i1, int i2)
	{
	      DocumentItem node1;
	      DocumentItem node2;
      
	      node1 = (DocumentItem)listofnodes.get(i1); 
	      node2 =(DocumentItem)listofnodes.get(i2); 
      	      
	      float xDiff = node1.getPosx() - node2.getPosx();
	      float yDiff = node1.getPosy() - node2.getPosy();
	      float zDiff = node1.getPosz() - node2.getPosz();
	      
	      return (float)Math.sqrt(xDiff*xDiff + yDiff*yDiff + zDiff*zDiff);
	      
   }
	/**
	 * Beregner plassering for node. lagres i en tabbel og retunerer
	 * ny energi for  bestemme om dette er en god plasssering eller ikke
	 * 
	 * @param index indeks noden som behandles
	 * @param dir tabell som inneholder plasseringen til noden
	 * @return
	 */
   protected float getEnergyDir(int index, float[] dir)
   {
      dir[0] = 0.0f;
      dir[1] = 0.0f;
      dir[2] = 0.0f;
      float energy = 0.0f;
      float dist;             // Euklidean distance to other node
//	      float avgDist = 0.0f;   // average Euklidean distance to other nodes
      float tmp;              // temporary, only for efficiency
      float simPow;           // temporary, only for efficiency
      float simPowDist;       // temporary, only for efficiency
      float distPow;          // temporary, only for efficiency
      float dir2 = 0.0f;      // second derivation of energy
      DocumentItem r1 = (DocumentItem) listofnodes.get(index), r2;

      for (int i = 0; i < numberofnodes; i++)
      {
         if (i == index)
         continue;
         r2 = (DocumentItem) listofnodes.get(i);
         dist = getDist(i, index);

         
         if (dist > 0.00001f)
         {
            simPow = equalitymatrix.getEquality(index, i);
            simPow *= simPow*simPow;
            simPowDist = simPow*dist;
            distPow = dist*dist;
            energy += simPowDist*distPow / 3 - (float)Math.log(dist);

            dir2 += simPowDist*2.0f + 1.0f / distPow;

            tmp = simPowDist - 1.0f / distPow;
            dir[0] += (r2.getPosx() - r1.getPosx()) * tmp;
            dir[1] += (r2.getPosy() - r1.getPosy()) * tmp;
            dir[2] += (r2.getPosz() - r1.getPosz()) * tmp;
         }
         else
         {
            energy = Float.MAX_VALUE;
         }
      }
      
      dir[0] /= dir2;
      dir[1] /= dir2;
      dir[2] /= dir2;
      
      return energy;
      
   }
	
}
